@@ -1,7 +1,7 @@
 /*
  * modules.c	Radius module support.
  *
- * Version:	$Id: modules.c,v 1.86.2.1.2.4 2006/04/07 17:53:25 aland Exp $
+ * Version:	$Id: modules.c,v 1.86.2.1.2.6 2007/04/08 06:56:57 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
 
-static const char rcsid[] = "$Id: modules.c,v 1.86.2.1.2.4 2006/04/07 17:53:25 aland Exp $";
+static const char rcsid[] = "$Id: modules.c,v 1.86.2.1.2.6 2007/04/08 06:56:57 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -589,15 +589,19 @@ static int load_component_section(CONF_SECTION *cs, int comp,
 		if (comp == RLM_COMPONENT_AUTH) {
 			DICT_VALUE *dval;
 			const char *modrefname = NULL;
+			int lineno = 0;
 
 			if (cp) {
 				modrefname = cf_pair_attr(cp);
+				lineno = cf_pair_lineno(cp);
 			} else {
 				modrefname = cf_section_name2(scs);
+				lineno = cf_section_lineno(scs);
+
 				if (!modrefname) {
 					radlog(L_ERR|L_CONS,
 					       "%s[%d] Failed to parse %s sub-section.\n",
-					       filename, cf_section_lineno(scs),
+					       filename, lineno,
 					       cf_section_name1(scs));
 					return -1;
 				}
@@ -610,7 +614,7 @@ static int load_component_section(CONF_SECTION *cs, int comp,
 				 *	recognize.  Die!
 				 */
 				radlog(L_ERR|L_CONS, "%s[%d] Unknown Auth-Type \"%s\" in %s section.",
-				       filename, cf_section_lineno(scs),
+				       filename, lineno,
 				       modrefname, component_names[comp]);
 				return -1;
 			}
@@ -726,9 +730,14 @@ int setup_modules(void)
 		for (comp = 0; comp < RLM_COMPONENT_COUNT; comp++) {
 			components[comp] = NULL;
 		}
-
 	} else {
-		detach_modules();
+		/*
+		 *	Else do NOT detach modules.  Instead, just
+		 *	forget about any module instances we may have
+		 *	previously loaded.  While this leaks memory,
+		 *	it's better than the server crashing.
+		 */
+		module_instance_list = NULL;
 	}
 
 	/*
