@@ -2,7 +2,7 @@
  * request_list.c	Hide the handling of the REQUEST list from
  *			the main server.
  *
- * Version:	$Id: request_list.c,v 1.40.2.2.2.1 2005/11/30 22:21:43 aland Exp $
+ * Version:	$Id: request_list.c,v 1.40.2.2.2.6 2007/03/05 14:15:32 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  *
  * Copyright 2003-2004  The FreeRADIUS server project
  */
-static const char rcsid[] = "$Id: request_list.c,v 1.40.2.2.2.1 2005/11/30 22:21:43 aland Exp $";
+static const char rcsid[] = "$Id: request_list.c,v 1.40.2.2.2.6 2007/03/05 14:15:32 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -251,6 +251,22 @@ static int proxy_cmp(const void *one, const void *two)
 }
 
 
+void rl_free(void)
+{
+	int i;
+
+	for (i = 0; i < 256; i++) {
+		while (request_list[i].first_request) {
+			rl_delete(request_list[i].first_request->req);
+		}
+	}
+
+	rbtree_free(request_tree);
+	rbtree_free(proxy_id_tree);
+	rbtree_free(proxy_tree);
+}
+
+
 /*
  *	Initialize the request list.
  */
@@ -490,7 +506,6 @@ void rl_delete(REQUEST *request)
 
 	request_free(&request);
 	request_list[id].request_count--;
-
 }
 
 /*
@@ -787,7 +802,10 @@ int rl_add_proxy(REQUEST *request)
 	/*
 	 *	There was no bit clear, which we had just checked above...
 	 */
-	rad_assert(proxy != -1);
+	if (proxy < 0) {
+		pthread_mutex_unlock(&proxy_mutex);
+		return 0;
+	}
 
 	/*
 	 *	Mark the Id as allocated, for thei Fd.
@@ -1201,25 +1219,13 @@ static int refresh_request(REQUEST *request, void *data)
 	request->proxy_try_count--;
 	request->proxy_next_try = info->now + mainconfig.proxy_retry_delay;
 
-	/* Fix up Acct-Delay-Time */
+	/*
+	 *	Don't restransmit accounting requests.
+	 *	Only the originating NAS should do this.
+	 */
 	if (request->proxy->code == PW_ACCOUNTING_REQUEST) {
-		VALUE_PAIR *delaypair;
-		delaypair = pairfind(request->proxy->vps, PW_ACCT_DELAY_TIME);
-
-		if (!delaypair) {
-			delaypair = paircreate(PW_ACCT_DELAY_TIME, PW_TYPE_INTEGER);
-			if (!delaypair) {
-				radlog(L_ERR|L_CONS, "no memory");
-				exit(1);
-			}
-			pairadd(&request->proxy->vps, delaypair);
-		}
-		delaypair->lvalue = info->now - request->proxy->timestamp;
-
-		/* Must recompile the valuepairs to wire format */
-		free(request->proxy->data);
-		request->proxy->data = NULL;
-	} /* proxy accounting request */
+		goto setup_timeout;
+	}
 
 	/*
 	 *  Assert that we have NOT seen the proxy reply yet.
