@@ -1,7 +1,7 @@
 /*
  * threads.c	request threading support
  *
- * Version:	$Id: threads.c,v 1.77.2.7 2005/10/17 18:38:07 aland Exp $
+ * Version:	$Id: threads.c,v 1.77.2.7.2.2 2007/02/14 15:44:22 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -58,7 +58,14 @@
 #include "conffile.h"
 
 static const char rcsid[] =
-"$Id: threads.c,v 1.77.2.7 2005/10/17 18:38:07 aland Exp $";
+"$Id: threads.c,v 1.77.2.7.2.2 2007/02/14 15:44:22 aland Exp $";
+
+#ifdef HAVE_OPENSSL_CRYPTO_H
+#include <openssl/crypto.h>
+#endif
+#ifdef HAVE_OPENSSL_ERR_H
+#include <openssl/err.h>
+#endif
 
 #define SEMAPHORE_LOCKED	(0)
 #define SEMAPHORE_UNLOCKED	(1)
@@ -98,7 +105,13 @@ typedef struct request_queue_t {
 } request_queue_t;
 
 
-#define MAX_WAITERS (256)
+typedef struct thread_fork_t {
+	pid_t		pid;
+	int		status;
+	int		exited;
+} thread_fork_t;
+
+
 /*
  *	A data structure to manage the thread pool.  There's no real
  *	need for a data structure, but it makes things conceptually
@@ -124,9 +137,7 @@ typedef struct THREAD_POOL {
 	 *	child PID's when we fork...
 	 */
 	pthread_mutex_t	wait_mutex;
-	int		wait_head;
-	int		wait_tail;
-	pid_t		wait[MAX_WAITERS];
+	lrad_hash_table_t *waiters;
 
 	/*
 	 *	All threads wait on this semaphore, for requests
@@ -164,6 +175,57 @@ static const CONF_PARSER thread_config[] = {
 };
 
 
+#ifdef HAVE_OPENSSL_CRYPTO_H
+
+/*
+ *	If we're linking against OpenSSL, then it is the
+ *	duty of the application, if it is multithreaded,
+ *	to provide OpenSSL with appropriate thread id
+ *	and mutex locking functions
+ *
+ *	Note: this only implements static callbacks.
+ *	OpenSSL does not use dynamic locking callbacks
+ *	right now, but may in the futiure, so we will have
+ *	to add them at some point.
+ */
+
+static  pthread_mutex_t *ssl_mutexes = NULL;
+
+
+static unsigned long ssl_id_function (void) {
+	return (unsigned long)pthread_self();
+}
+
+static void ssl_locking_function (int mode, int n, const char *file, int line) {
+	if (mode & CRYPTO_LOCK) {
+		pthread_mutex_lock(&(ssl_mutexes[n]));
+	} else {
+		pthread_mutex_unlock(&(ssl_mutexes[n]));
+	}
+}
+
+static int setup_ssl_mutexes (void) {
+	int i;
+
+	ssl_mutexes = (pthread_mutex_t *)rad_malloc(CRYPTO_num_locks() * sizeof(pthread_mutex_t));
+	if(!ssl_mutexes) {
+		radlog(L_ERR, "Error allocating memory for SSL mutexes!");
+		return 0;
+	}
+
+	for(i = 0; i < CRYPTO_num_locks(); i++) {
+		pthread_mutex_init(&(ssl_mutexes[i]), NULL);
+	}
+
+	CRYPTO_set_id_callback(ssl_id_function);
+	CRYPTO_set_locking_callback(ssl_locking_function);
+
+	return 1;
+}
+
+#endif
+
+
 /*
  *	We don't want to catch SIGCHLD for a host of reasons.
  *
@@ -182,34 +244,25 @@ static const CONF_PARSER thread_config[] = {
  */
 static void reap_children(void)
 {
-	if (thread_pool.wait_head != thread_pool.wait_tail) {
-		int num;
-		
-		pthread_mutex_lock(&thread_pool.wait_mutex);
-		for (num = ((thread_pool.wait_tail + MAX_WAITERS) - thread_pool.wait_head) % MAX_WAITERS;
-		     num != 0;
-		     num--) {
-			pid_t pid = thread_pool.wait[thread_pool.wait_head];
-			
-			thread_pool.wait_head++;
-			thread_pool.wait_head %= MAX_WAITERS;
-			
-			/*
-			 *	Child is still alive: move it to the tail.
-			 */
-			if (waitpid(pid, NULL, WNOHANG) == 0) {
-				if (((thread_pool.wait_tail + 1) % MAX_WAITERS)
-				    == thread_pool.wait_head) {
-					rad_assert(0 == 1);
-				}
+	pid_t pid;
+	int status;
+	thread_fork_t mytf, *tf;
 
-				thread_pool.wait[thread_pool.wait_tail] = pid;
-				thread_pool.wait_tail++;
-				thread_pool.wait_tail %= MAX_WAITERS;
-			} /* else no child, or was already reaped */
-		}
-		pthread_mutex_unlock(&thread_pool.wait_mutex);
-	}
+	pthread_mutex_lock(&thread_pool.wait_mutex);
+
+	do {
+		pid = waitpid(0, &status, WNOHANG);
+		if (pid <= 0) break;
+
+		mytf.pid = pid;
+		tf = lrad_hash_table_finddata(thread_pool.waiters, &mytf);
+		if (!tf) continue;
+		
+		tf->status = status;
+		tf->exited = 1;
+	} while (lrad_hash_table_num_elements(thread_pool.waiters) > 0);
+
+	pthread_mutex_unlock(&thread_pool.wait_mutex);
 }
 
 
@@ -528,6 +581,15 @@ static void *request_handler_thread(void *arg)
 
 	DEBUG2("Thread %d exiting...", self->thread_num);
 
+#ifdef HAVE_OPENSSL_ERR_H
+	/*
+	 *	If we linked with OpenSSL, the application
+	 *	must remove the thread's error queue before
+	 *	exiting to prevent memory leaks.
+	 */
+	ERR_remove_state(0);
+#endif
+
 	/*
 	 *  Do this as the LAST thing before exiting.
 	 */
@@ -692,6 +754,22 @@ int total_active_threads(void)
 	return (rcode);
 }
 
+
+static uint32_t pid_hash(const void *data)
+{
+	const thread_fork_t *tf = data;
+
+	return lrad_hash(&tf->pid, sizeof(tf->pid));
+}
+
+static int pid_cmp(const void *one, const void *two)
+{
+	const thread_fork_t *a = one;
+	const thread_fork_t *b = two;
+
+	return (a->pid - b->pid);
+}
+
 /*
  *	Allocate the thread pool, and seed it with an initial number
  *	of threads.
@@ -721,12 +799,22 @@ int thread_pool_init(void)
 		thread_pool.max_thread_num = 1;
 		thread_pool.cleanup_delay = 5;
 
-		thread_pool.wait_head = thread_pool.wait_tail = 0;
 		if ((pthread_mutex_init(&thread_pool.wait_mutex,NULL) != 0)) {
 			radlog(L_ERR, "FATAL: Failed to initialize mutex: %s",
 			       strerror(errno));
 			exit(1);
 		}		
+
+		/*
+		 *	Create the hash table of child PID's
+		 */
+		thread_pool.waiters = lrad_hash_table_create(pid_hash,
+							     pid_cmp,
+							     free);
+		if (!thread_pool.waiters) {
+			radlog(L_ERR, "FATAL: Failed to set up wait hash");
+			exit(1);
+		}
 	}
 
 	pool_cf = cf_section_find("thread");
@@ -771,6 +859,18 @@ int thread_pool_init(void)
 	memset(thread_pool.queue, 0, (sizeof(*thread_pool.queue) *
 				      thread_pool.queue_size));
 
+#ifdef HAVE_OPENSSL_CRYPTO_H
+	/*
+	 *	If we're linking with OpenSSL too, then we need
+	 *	to set up the mutexes and enable the thread callbacks.
+	 */
+	if(!setup_ssl_mutexes()) {
+		radlog(L_ERR, "FATAL: Failed to set up SSL mutexes");
+		exit(1);
+	}
+#endif
+
+
 	/*
 	 *	Create a number of waiting threads.
 	 *
@@ -982,43 +1082,39 @@ int thread_pool_clean(time_t now)
 /*
  *	Thread wrapper for fork().
  */
-pid_t rad_fork(int exec_wait)
+pid_t rad_fork(void)
 {
 	pid_t child_pid;
 
-	if (exec_wait) return fork();
+	if (!pool_initialized) return fork();
 
-	/*
-	 *	Ensure that children are reaped always.
-	 */
-	reap_children();
+	reap_children();	/* be nice to non-wait thingies */
 
-	/*
-	 *	Lock the mutex.
-	 */
-	pthread_mutex_lock(&thread_pool.wait_mutex);
-
-	/*
-	 *	No room to save the PID: die.
-	 */
-	if (((thread_pool.wait_tail + 1) % MAX_WAITERS)
-	    == thread_pool.wait_head) {
-		rad_assert(0 == 1);
+	if (lrad_hash_table_num_elements(thread_pool.waiters) >= 1024) {
+		return -1;
 	}
 
 	/*
 	 *	Fork & save the PID for later reaping.
 	 */
 	child_pid = fork();
-	if (child_pid != 0) {
-		thread_pool.wait[thread_pool.wait_tail] = child_pid;
-		thread_pool.wait_tail++;
-		thread_pool.wait_tail %= MAX_WAITERS;
+	if (child_pid > 0) {
+		int rcode;
+		thread_fork_t *tf;
 
-		/*
-		 *	Unlock the mutex.
-		 */
+		tf = rad_malloc(sizeof(*tf));
+		memset(tf, 0, sizeof(*tf));
+		
+		tf->pid = child_pid;
+
+		pthread_mutex_lock(&thread_pool.wait_mutex);
+		rcode = lrad_hash_table_insert(thread_pool.waiters, tf);
 		pthread_mutex_unlock(&thread_pool.wait_mutex);
+
+		if (!rcode) {
+			radlog(L_ERR, "Failed to store PID, creating what will be a zombie process %d",
+			       (int) child_pid);
+		}
 	}
 
 	/*
@@ -1027,15 +1123,48 @@ pid_t rad_fork(int exec_wait)
 	return child_pid;
 }
 
-
 /*
- *	We may not need this any more...
+ *	Wait 10 seconds at most for a child to exit, then give up.
  */
-pid_t rad_waitpid(pid_t pid, int *status, int options)
+pid_t rad_waitpid(pid_t pid, int *status)
 {
-	reap_children();	/* be nice to non-wait thingies */
-	return waitpid(pid, status, options);
+	int i;
+	thread_fork_t mytf, *tf;
+
+	if (!pool_initialized) return waitpid(pid, status, 0);
+
+	if (pid <= 0) return -1;
+
+	mytf.pid = pid;
+
+	pthread_mutex_lock(&thread_pool.wait_mutex);
+	tf = lrad_hash_table_finddata(thread_pool.waiters, &mytf);
+	pthread_mutex_unlock(&thread_pool.wait_mutex);
+
+	if (!tf) return -1;
 	
+	for (i = 0; i < 100; i++) {
+		reap_children();
+		
+		if (tf->exited) {
+			*status = tf->status;
+
+			pthread_mutex_lock(&thread_pool.wait_mutex);
+			lrad_hash_table_delete(thread_pool.waiters, &mytf);
+			pthread_mutex_unlock(&thread_pool.wait_mutex);
+			return pid;
+		}
+		usleep(100000);
+	}
+	
+	/*
+	 *	10 seconds have passed, give up on the child.
+	 */
+	pthread_mutex_lock(&thread_pool.wait_mutex);
+	lrad_hash_table_delete(thread_pool.waiters, &mytf);
+	pthread_mutex_unlock(&thread_pool.wait_mutex);
+
+	return 0;
 }
 
 #endif
