@@ -2,7 +2,7 @@
  * xlat.c	Translate strings.  This is the first version of xlat
  * 		incorporated to RADIUS
  *
- * Version:	$Id: xlat.c,v 1.72.2.7.2.2 2006/07/13 16:40:54 aland Exp $
+ * Version:	$Id: xlat.c,v 1.72.2.7.2.4 2007/02/09 15:56:13 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 static const char rcsid[] =
-"$Id: xlat.c,v 1.72.2.7.2.2 2006/07/13 16:40:54 aland Exp $";
+"$Id: xlat.c,v 1.72.2.7.2.4 2007/02/09 15:56:13 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -411,6 +411,16 @@ void xlat_unregister(const char *module, RAD_XLAT_FUNC func)
 }
 
 
+/*
+ *	De-register all xlat functions,
+ *	used mainly for debugging.
+ */
+void xlat_free(void)
+{
+	rbtree_free(xlat_root);
+}
+
+
 /*
  *	Decode an attribute name into a string.
  */
@@ -418,7 +428,7 @@ static void decode_attribute(const char **from, char **to, int freespace,
 			     int *open, REQUEST *request,
 			     RADIUS_ESCAPE_STRING func)
 {
-	char attrname[256];
+	char attrname[4096];
 	const char *p;
 	char *q, *pa;
 	int stop=0, found=0, retlen=0;
