@@ -1,7 +1,7 @@
 /*
  * rlm_counter.c
  *
- * Version:  $Id: rlm_counter.c,v 1.32.2.5 2003/10/04 00:20:54 phampson Exp $
+ * Version:  $Id: rlm_counter.c,v 1.43 2004/02/26 19:04:28 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -19,7 +19,7 @@
  *
  * Copyright 2001  The FreeRADIUS server project
  * Copyright 2001  Alan DeKok <aland@ox.org>
- * Copyright 2001,2002  Kostas Kalevras <kkalev@noc.ntua.gr>
+ * Copyright 2001-3  Kostas Kalevras <kkalev@noc.ntua.gr>
  */
 
 #include "config.h"
@@ -56,7 +56,7 @@
 
 #define UNIQUEID_MAX_LEN 32
 
-static const char rcsid[] = "$Id: rlm_counter.c,v 1.32.2.5 2003/10/04 00:20:54 phampson Exp $";
+static const char rcsid[] = "$Id: rlm_counter.c,v 1.43 2004/02/26 19:04:28 aland Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -130,8 +130,10 @@ static int counter_detach(void *instance);
 /*
  *	See if the counter matches.
  */
-static int counter_cmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check,
-		VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
+static int counter_cmp(void *instance,
+		       REQUEST *req UNUSED,
+		       VALUE_PAIR *request, VALUE_PAIR *check,
+		       VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
 {
 	rlm_counter_t *data = (rlm_counter_t *) instance;
 	datum key_datum;
@@ -141,6 +143,7 @@ static int counter_cmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_
 
 	check_pairs = check_pairs; /* shut the compiler up */
 	reply_pairs = reply_pairs;
+	req = req;
 
 	/*
 	 *	Find the key attribute.
@@ -171,10 +174,10 @@ static int add_defaults(rlm_counter_t *data)
 	datum time_datum;
 	const char *default1 = "DEFAULT1";
 	const char *default2 = "DEFAULT2";
-	
+
 	DEBUG2("rlm_counter: add_defaults: Start");
 
-	key_datum.dptr = (const char *) default1;
+	key_datum.dptr = (char *) default1;
 	key_datum.dsize = strlen(default1);
 	time_datum.dptr = (char *) &data->reset_time;
 	time_datum.dsize = sizeof(time_t);
@@ -186,7 +189,7 @@ static int add_defaults(rlm_counter_t *data)
 	}
 	DEBUG2("rlm_counter: DEFAULT1 set to %d",(int)data->reset_time);
 
-	key_datum.dptr = (const char *) default2;
+	key_datum.dptr = (char *) default2;
 	key_datum.dsize = strlen(default2);
 	time_datum.dptr = (char *) &data->last_reset;
 	time_datum.dsize = sizeof(time_t);
@@ -229,7 +232,7 @@ static int reset_db(rlm_counter_t *data)
 	 */
 	ret = add_defaults(data);
 	if (ret != RLM_MODULE_OK)
-		return ret;	
+		return ret;
 
 	DEBUG2("rlm_counter: reset_db ended");
 
@@ -242,8 +245,10 @@ static int find_next_reset(rlm_counter_t *data, time_t timeval)
 	unsigned int num=1;
 	char last = 0;
 	struct tm *tm, s_tm;
+	char sCurrentTime[40], sNextTime[40];
 
 	tm = localtime_r(&timeval, &s_tm);
+	strftime(sCurrentTime, sizeof(sCurrentTime),"%Y-%m-%d %H:%M:%S",tm);
 	tm->tm_sec = tm->tm_min = 0;
 
 	if (data->reset == NULL)
@@ -292,8 +297,9 @@ static int find_next_reset(rlm_counter_t *data, time_t timeval)
 			data->reset);
 		return -1;
 	}
-	DEBUG2("rlm_counter: Current Time: %d, Next reset %d", 
-		(int)timeval,(int)data->reset_time);
+	strftime(sNextTime, sizeof(sNextTime),"%Y-%m-%d %H:%M:%S",tm);
+	DEBUG2("rlm_counter: Current Time: %d [%s], Next reset %d [%s]",
+		(int)timeval,sCurrentTime,(int)data->reset_time,sNextTime);
 
 	return ret;
 }
@@ -322,7 +328,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	datum time_datum;
 	const char *default1 = "DEFAULT1";
 	const char *default2 = "DEFAULT2";
-	
+
 	/*
 	 *	Set up a storage area for instance data
 	 */
@@ -344,7 +350,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	cache_size = data->cache_size;
 
 	/*
-	 *	Discover the attribute number of the key. 
+	 *	Discover the attribute number of the key.
 	 */
 	if (data->key_name == NULL) {
 		radlog(L_ERR, "rlm_counter: 'key' must be set.");
@@ -359,9 +365,9 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 		return -1;
 	}
 	data->key_attr = dattr->attr;
-	
+
 	/*
-	 *	Discover the attribute number of the counter. 
+	 *	Discover the attribute number of the counter.
 	 */
 	if (data->count_attribute == NULL) {
 		radlog(L_ERR, "rlm_counter: 'count-attribute' must be set.");
@@ -428,7 +434,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 			return -1;
 		}
 		data->service_val = dval->value;
-	}	
+	}
 
 	/*
 	 * Find when to reset the database.
@@ -477,7 +483,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	 * If DEFAULT1 and DEFAULT2 do not exist (new database) we add them to the database
 	 */
 
-	key_datum.dptr = (const char *)default1;
+	key_datum.dptr = (char *)default1;
 	key_datum.dsize = strlen(default1);
 
 	time_datum = gdbm_fetch(data->gdbm, key_datum);
@@ -498,10 +504,10 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 		}
 		else
 			data->reset_time = next_reset;
-		key_datum.dptr = (const char *)default2;
+		key_datum.dptr = (char *)default2;
 		key_datum.dsize = strlen(default2);
 
-		time_datum = gdbm_fetch(data->gdbm, key_datum);	
+		time_datum = gdbm_fetch(data->gdbm, key_datum);
 		if (time_datum.dptr != NULL){
 			memcpy(&data->last_reset, time_datum.dptr, sizeof(time_t));
 			free(time_datum.dptr);
@@ -528,7 +534,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	pthread_mutex_init(&data->mutex, NULL);
 
 	*instance = data;
-	
+
 	return 0;
 }
 
@@ -567,6 +573,7 @@ static int counter_accounting(void *instance, REQUEST *request)
 	if (data->reset_time && (data->reset_time <= request->timestamp)) {
 		int ret;
 
+		DEBUG("rlm_counter: Time to reset the database.");
 		data->last_reset = data->reset_time;
 		find_next_reset(data,request->timestamp);
 		pthread_mutex_lock(&data->mutex);
@@ -583,7 +590,7 @@ static int counter_accounting(void *instance, REQUEST *request)
 			DEBUG("rlm_counter: Could not find Service-Type attribute in the request. Returning NOOP.");
 			return RLM_MODULE_NOOP;
 		}
-		if (proto_vp->lvalue != data->service_val){
+		if ((unsigned)proto_vp->lvalue != data->service_val){
 			DEBUG("rlm_counter: This Service-Type is not allowed. Returning NOOP.");
 			return RLM_MODULE_NOOP;
 		}
@@ -594,13 +601,14 @@ static int counter_accounting(void *instance, REQUEST *request)
 	 */
 	key_vp = pairfind(request->packet->vps, PW_ACCT_DELAY_TIME);
 	if (key_vp != NULL){
-		if (key_vp->lvalue != 0 && (request->timestamp - key_vp->lvalue) < data->last_reset){
+		if (key_vp->lvalue != 0 &&
+		    (request->timestamp - key_vp->lvalue) < data->last_reset){
 			DEBUG("rlm_counter: This packet is too old. Returning NOOP.");
 			return RLM_MODULE_NOOP;
 		}
 	}
 
-	
+
 
 	/*
 	 *	Look for the key.  User-Name is special.  It means
@@ -624,23 +632,26 @@ static int counter_accounting(void *instance, REQUEST *request)
 	key_datum.dptr = key_vp->strvalue;
 	key_datum.dsize = key_vp->length;
 
+	DEBUG("rlm_counter: Searching the database for key '%s'",key_vp->strvalue);
 	pthread_mutex_lock(&data->mutex);
 	count_datum = gdbm_fetch(data->gdbm, key_datum);
 	pthread_mutex_unlock(&data->mutex);
 	if (count_datum.dptr == NULL){
+		DEBUG("rlm_counter: Could not find the requested key in the database.");
 		counter.user_counter = 0;
 		if (uniqueid_vp != NULL)
-			strncpy(uniqueid_vp->strvalue,counter.uniqueid,UNIQUEID_MAX_LEN - 1);
+			strncpy(counter.uniqueid,uniqueid_vp->strvalue,UNIQUEID_MAX_LEN - 1);
 		else
 			memset((char *)counter.uniqueid,0,UNIQUEID_MAX_LEN);
 	}
 	else{
+		DEBUG("rlm_counter: Key found.");
 		memcpy(&counter, count_datum.dptr, sizeof(rad_counter));
 		free(count_datum.dptr);
 		if (counter.uniqueid)
 			DEBUG("rlm_counter: Counter Unique ID = '%s'",counter.uniqueid);
-		if (uniqueid_vp != NULL){ 
-			if (counter.uniqueid != NULL && 
+		if (uniqueid_vp != NULL){
+			if (counter.uniqueid != NULL &&
 				strncmp(uniqueid_vp->strvalue,counter.uniqueid, UNIQUEID_MAX_LEN - 1) == 0){
 				DEBUG("rlm_counter: Unique IDs for user match. Droping the request.");
 				return RLM_MODULE_NOOP;
@@ -680,9 +691,10 @@ static int counter_accounting(void *instance, REQUEST *request)
 	}
 
 	DEBUG("rlm_counter: User=%s, New Counter=%d.",request->username->strvalue,counter.user_counter);
-	count_datum.dptr = (rad_counter *) &counter;
+	count_datum.dptr = (char *) &counter;
 	count_datum.dsize = sizeof(rad_counter);
 
+	DEBUG("rlm_counter: Storing new value in database.");
 	pthread_mutex_lock(&data->mutex);
 	rcode = gdbm_store(data->gdbm, key_datum, count_datum, GDBM_REPLACE);
 	pthread_mutex_unlock(&data->mutex);
@@ -691,6 +703,7 @@ static int counter_accounting(void *instance, REQUEST *request)
 				data->filename, gdbm_strerror(gdbm_errno));
 		return RLM_MODULE_FAIL;
 	}
+	DEBUG("rlm_counter: New value stored successfully.");
 
 	return RLM_MODULE_OK;
 }
@@ -762,23 +775,29 @@ static int counter_authorize(void *instance, REQUEST *request)
 	 */
 
 	counter.user_counter = 0;
-	
+
+	DEBUG("rlm_counter: Searching the database for key '%s'",key_vp->strvalue);
 	pthread_mutex_lock(&data->mutex);
 	count_datum = gdbm_fetch(data->gdbm, key_datum);
 	pthread_mutex_unlock(&data->mutex);
 	if (count_datum.dptr != NULL){
+		DEBUG("rlm_counter: Key Found.");
 		memcpy(&counter, count_datum.dptr, sizeof(rad_counter));
 		free(count_datum.dptr);
 	}
+	else
+		DEBUG("rlm_counter: Could not find the requested key in the database.");
 
 	/*
 	 * Check if check item > counter
 	 */
+	DEBUG("rlm_counter: Check item = %d, Count = %d",check_vp->lvalue,counter.user_counter);
 	res=check_vp->lvalue - counter.user_counter;
 	if (res > 0) {
+		DEBUG("rlm_counter: res is greater than zero");
 		if (data->count_attr == PW_ACCT_SESSION_TIME) {
 			/*
-			 * Do the following only if the count attribute is 
+			 * Do the following only if the count attribute is
 			 * AcctSessionTime
 			 */
 
@@ -838,7 +857,7 @@ static int counter_authorize(void *instance, REQUEST *request)
 
 		snprintf(module_fmsg,sizeof(module_fmsg), "rlm_counter: Maximum %s usage time reached", data->reset);
 		module_fmsg_vp = pairmake("Module-Failure-Message", module_fmsg, T_OP_EQ);
-		pairadd(&request->packet->vps, module_fmsg_vp);	
+		pairadd(&request->packet->vps, module_fmsg_vp);
 
 		ret=RLM_MODULE_REJECT;
 
@@ -879,7 +898,7 @@ static int counter_detach(void *instance)
  *	is single-threaded.
  */
 module_t rlm_counter = {
-	"Counter",	
+	"Counter",
 	RLM_TYPE_THREAD_SAFE,		/* type */
 	NULL,				/* initialization */
 	counter_instantiate,		/* instantiation */
