@@ -1,7 +1,7 @@
 /*
  * rlm_eap_ttls.c  contains the interfaces that are called from eap
  *
- * Version:     $Id: ttls.c,v 1.17.2.2.2.2 2006/07/17 19:07:37 aland Exp $
+ * Version:     $Id: ttls.c,v 1.17.2.2.2.6 2007/04/08 06:29:48 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -259,6 +259,7 @@ static VALUE_PAIR *diameter2vp(SSL *ssl,
 				DEBUG2("  rlm_eap_ttls: Invalid length attribute %d",
 				       attr);
 				pairfree(&first);
+				pairfree(&vp);
 				return NULL;
 			}
 			memcpy(&vp->lvalue, data, vp->length);
@@ -274,6 +275,7 @@ static VALUE_PAIR *diameter2vp(SSL *ssl,
 				DEBUG2("  rlm_eap_ttls: Invalid length attribute %d",
 				       attr);
 				pairfree(&first);
+				pairfree(&vp);
 				return NULL;
 			}
 		  memcpy(&vp->lvalue, data, vp->length);
@@ -344,21 +346,21 @@ static VALUE_PAIR *diameter2vp(SSL *ssl,
 			    (vp->length > 16)) {
 				DEBUG2("  TTLS: Tunneled challenge has invalid length");
 				pairfree(&first);
+				pairfree(&vp);
 				return NULL;
 
 			} else {
-				int i;
 				uint8_t	challenge[16];
 
 				eapttls_gen_challenge(ssl, challenge,
 						      sizeof(challenge));
 
-				for (i = 0; i < vp->length; i++) {
-					if (challenge[i] != vp->strvalue[i]) {
-						DEBUG2("  TTLS: Tunneled challenge is incorrect");
-						pairfree(&first);
-						return NULL;
-					}
+				if (memcmp(challenge, vp->strvalue,
+					   vp->length) != 0) {
+					DEBUG2("  TTLS: Tunneled challenge is incorrect");
+					pairfree(&first);
+					pairfree(&vp);
+					return NULL;
 				}
 			}
 			break;
@@ -608,19 +610,27 @@ static int process_reply(EAP_HANDLER *handler, tls_session_t *tls_session,
 		vp = NULL;
 		pairmove2(&vp, &reply->vps, PW_MSCHAP2_SUCCESS);
 		if (vp) {
-#if 1
-			/*
-			 *	FIXME: Tunneling MS-CHAP2-Success causes
-			 *	the only client we have access to, to die.
-			 *
-			 *	We don't want that...
-			 */
-			pairfree(&vp);
-#else
 			DEBUG2("  TTLS: Got MS-CHAP2-Success, tunneling it to the client in a challenge.");
 			rcode = RLM_MODULE_HANDLED;
 			t->authenticated = TRUE;
-#endif
+			
+			/*
+			 *	Delete MPPE keys & encryption policy.  We don't
+			 *	want these here.
+			 */
+			pairdelete(&reply->vps, ((311 << 16) | 7));
+			pairdelete(&reply->vps, ((311 << 16) | 8));
+			pairdelete(&reply->vps, ((311 << 16) | 16));
+			pairdelete(&reply->vps, ((311 << 16) | 17));
+			
+			/*
+			 *	Use the tunneled reply, but not now.
+			 */
+			if (t->use_tunneled_reply) {
+				t->reply = reply->vps;
+				reply->vps = NULL;
+			}
+
 		} else { /* no MS-CHAP2-Success */
 			/*
 			 *	Can only have EAP-Message if there's
