@@ -1,7 +1,7 @@
 /*
  * rlm_exe.c
  *
- * Version:	$Id: rlm_exec.c,v 1.11 2004/02/26 19:04:32 aland Exp $
+ * Version:	$Id: rlm_exec.c,v 1.11.4.1 2007/04/07 22:47:11 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #include "modules.h"
 #include "conffile.h"
 
-static const char rcsid[] = "$Id: rlm_exec.c,v 1.11 2004/02/26 19:04:32 aland Exp $";
+static const char rcsid[] = "$Id: rlm_exec.c,v 1.11.4.1 2007/04/07 22:47:11 aland Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -276,7 +276,7 @@ static int exec_instantiate(CONF_SECTION *conf, void **instance)
 static int exec_dispatch(void *instance, REQUEST *request)
 {
 	int result;
-	VALUE_PAIR **input_pairs, **output_pairs;
+	VALUE_PAIR **input_pairs, **output_pairs, *in;
 	VALUE_PAIR *answer;
 	rlm_exec_t *inst = (rlm_exec_t *) instance;
 
@@ -314,8 +314,11 @@ static int exec_dispatch(void *instance, REQUEST *request)
 	 *	It points to the attribute list, but the attribute
 	 *	list is empty.
 	 */
-	if (input_pairs && !*input_pairs) {
+	if (!input_pairs || (input_pairs && !*input_pairs)) {
 		DEBUG2("rlm_exec (%s): WARNING! Input pairs are empty.  No attributes will be passed to the script", inst->xlat_name);
+		in = NULL;
+	} else {
+		in = *input_pairs;
 	}
 
 	/*
@@ -331,7 +334,7 @@ static int exec_dispatch(void *instance, REQUEST *request)
 	 */
 	result = radius_exec_program(inst->program, request,
 				     inst->wait, NULL, 0,
-				     *input_pairs, &answer);
+				     in, &answer);
 	if (result != 0) {
 		radlog(L_ERR, "rlm_exec (%s): External script failed",
 		       inst->xlat_name);
