@@ -157,7 +157,7 @@
  *	  a corresponding ldap_groupcmp function
  *	- Small change to ldap_get_conn to fix problems on some platforms
  */
-static const char rcsid[] = "$Id: rlm_ldap.c,v 1.122.2.8.2.7 2006/05/08 23:45:02 aland Exp $";
+static const char rcsid[] = "$Id: rlm_ldap.c,v 1.122.2.8.2.8 2007/04/08 07:14:45 aland Exp $";
 
 #include "autoconf.h"
 
@@ -848,7 +848,9 @@ static int ldap_escape_func(char *out, int outlen, const char *in)
 		/*
 		 *	Encode unsafe characters.
 		 */
-		if (strchr("*=\\,()", *in)) {
+		if (((len == 0) &&
+		    ((in[0] == ' ') || (in[0] == '#'))) ||
+		    (strchr(",+\"\\<>;*=()", *in))) {
 			static const char hex[] = "0123456789abcdef";
 
 			/*
