@@ -2,7 +2,7 @@
  *  sql.c		rlm_sql - FreeRADIUS SQL Module
  *		Main code directly taken from ICRADIUS
  *
- * Version:	$Id: sql.c,v 1.79.2.3 2005/08/26 00:37:47 aland Exp $
+ * Version:	$Id: sql.c,v 1.79.2.3.2.2 2007/03/26 09:46:28 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -496,7 +496,9 @@ int rlm_sql_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query)
 
 	if (ret == SQL_DOWN) {
 	        /* close the socket that failed */
-	        (inst->module->sql_close)(sqlsocket, inst->config);
+		if (sqlsocket->state == sockconnected) {
+			(inst->module->sql_close)(sqlsocket, inst->config);
+		}
 
 		/* reconnect the socket */
 		if (connect_single_socket(sqlsocket, inst) < 0) {
@@ -539,7 +541,9 @@ int rlm_sql_select_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query)
 
 	if (ret == SQL_DOWN) {
 	        /* close the socket that failed */
-	        (inst->module->sql_close)(sqlsocket, inst->config);
+		if (sqlsocket->state == sockconnected) {
+			(inst->module->sql_close)(sqlsocket, inst->config);
+		}
 
 		/* reconnect the socket */
 		if (connect_single_socket(sqlsocket, inst) < 0) {
